import java.awt.*;

/**
 * A layout manager that resizes all components to fill
 * the entire area within its container (minus the insets).
 * 
 * @author David Foster <davidfstr@gmail.com>
 */
public class FillLayout implements LayoutManager {
	public void addLayoutComponent(String name, Component comp) {}
	
	public void layoutContainer(Container parent) {
		Dimension size = parent.getSize();
		subtractInsetsFromDimension(parent.getInsets(), size);
		for (int i=0, n=parent.getComponentCount(); i<n; i++) {
			Component comp = parent.getComponent(i);
			comp.setLocation(
				parent.getInsets().left,
				parent.getInsets().top);
			comp.setSize(size);
		}
	}
	
	public Dimension preferredLayoutSize(Container parent) {
		Dimension fullSize = new Dimension(0, 0);
		for (int i=0, n=parent.getComponentCount(); i<n; i++) {
			Component comp = parent.getComponent(i);
			Dimension minSize = comp.getPreferredSize();
			if (minSize.width > fullSize.width)
				fullSize.width = minSize.width;
			if (minSize.height > fullSize.height)
				fullSize.height = minSize.height;
		}
		addInsetsToDimension(parent.getInsets(), fullSize);
		return fullSize;
	}
	
	public Dimension minimumLayoutSize(Container parent) {
		Dimension fullSize = new Dimension(0, 0);
		for (int i=0, n=parent.getComponentCount(); i<n; i++) {
			Component comp = parent.getComponent(i);
			Dimension minSize = comp.getMinimumSize();
			if (minSize.width > fullSize.width)
				fullSize.width = minSize.width;
			if (minSize.height > fullSize.height)
				fullSize.height = minSize.height;
		}
		addInsetsToDimension(parent.getInsets(), fullSize);
		return fullSize;
	}
	
	private void addInsetsToDimension(Insets insets, Dimension dim) {
		dim.width += insets.left;
		dim.width += insets.right;
		dim.height += insets.top;
		dim.height += insets.bottom;
	}
	
	private void subtractInsetsFromDimension(Insets insets, Dimension dim) {
		dim.width -= insets.left;
		dim.width -= insets.right;
		dim.height -= insets.top;
		dim.height -= insets.bottom;
	}
	
	public void removeLayoutComponent(Component comp) {}
}
